/* -*-c-*-
 *
 *  $Id: getpw.c,v 1.3 2002/07/20 23:35:46 scip Exp $
 * 
 *  This file  is  part of the  MD5PW  password  generator  tool.
 *
 *  By  accessing  this software,  MD5PW, you  are  duly informed
 *  of and agree to be  bound by the  conditions  described below
 *  in this notice:
 *
 *  This software product,  MD5PW,  is developed by Thomas Linden
 *  and   copyrighted (C) 1999-2002   by  Thomas Linden, with all
 *  rights reserved.
 *
 *  There  is no charge for MD5PW software.  You can redistribute
 *  it and/or modify it under the terms of the GNU General Public
 *  License, which is incorporated by reference herein.
 *
 *  MD5PW is distributed WITHOUT ANY WARRANTY, IMPLIED OR EXPRESS,
 *  OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE or that
 *  the use of it will not infringe on any third party's intellec-
 *  tual property rights.
 *
 *  You should have received a copy of the GNU General Public
 *  License along with MD5PW.  Copies can also be obtained from:
 *
 *    http://www.gnu.org/copyleft/gpl.html
 *
 *  or by writing to:
 *
 *  Free Software Foundation, Inc.
 *  59 Temple Place, Suite 330
 *  Boston, MA 02111-1307
 *  USA
 *
 *  Or contact:
 *
 *   "Thomas Linden" <tom@daemon.de>
 *
 *
 */


#include "getpw.h"


#ifdef USETERM


  /*
   * stuff stolen from sshd (somewhat modified)
   */

/* Saved old terminal mode for read_passphrase. */
#ifdef USING_TERMIOS
static struct termios saved_tio;
#endif
#ifdef USING_SGTTY
static struct sgttyb saved_tio;
#endif



/* Old interrupt signal handler for read_passphrase. */
static RETSIGTYPE (*old_handler)(int sig) = NULL;

/* Interrupt signal handler for read_passphrase. */
RETSIGTYPE intr_handler(int sig)
{
  /* Restore terminal modes. */
#ifdef USING_TERMIOS
  tcsetattr(fileno(stdin), TCSANOW, &saved_tio);
#endif
#ifdef USING_SGTTY
  ioctl(fileno(stdin), TIOCSETP, &saved_tio);
#endif
  /* Restore the old signal handler. */
  signal(sig, old_handler);
  /* Resend the signal, with the old handler. */
  kill(getpid(), sig);
}


char *read_passphrase( const char *prompt, char *cp ) {


    char buf[1024];
#ifdef USING_TERMIOS
    struct termios tio;
#endif
#ifdef USING_SGTTY
    struct sgttyb tio;
#endif
    FILE *f;


    fprintf(stderr, "%s", prompt);
    fflush(stderr);

    f = fopen("/dev/tty", "r");
    if (!f) {
      fprintf(stderr, "Failed to open /dev/tty!\n");
      exit(-1);
    }
    
  /* Get terminal modes. */
#ifdef USING_TERMIOS
  tcgetattr(fileno(f), &tio);
#endif
#ifdef USING_SGTTY
  ioctl(fileno(f), TIOCGETP, &tio);
#endif
  saved_tio = tio;
  /* Save signal handler and set the new handler. */
  old_handler = signal(SIGINT, intr_handler);

  /* Set new terminal modes disabling all echo. */
#ifdef USING_TERMIOS
  tio.c_lflag &= ~(ECHO | ECHOE | ECHOK | ECHONL);
  tcsetattr(fileno(f), TCSANOW, &tio);
#endif
#ifdef USING_SGTTY
  tio.sg_flags &= ~(ECHO);
  ioctl(fileno(f), TIOCSETP, &tio);
#endif

  /* Read the passphrase from the terminal. */
  if (fgets(buf, sizeof(buf), f) == NULL)
    {
      /* Got EOF.  Just exit. */
      /* Restore terminal modes. */
#ifdef USING_TERMIOS
      tcsetattr(fileno(f), TCSANOW, &saved_tio);
#endif
#ifdef USING_SGTTY
      ioctl(fileno(f), TIOCSETP, &saved_tio);
#endif
      /* Restore the signal handler. */
      signal(SIGINT, old_handler);
      /* Close the file. */
      if (f != stdin)
        fclose(f);
      exit(1);
    }
  /* Restore terminal modes. */
#ifdef USING_TERMIOS
  tcsetattr(fileno(f), TCSANOW, &saved_tio);
#endif
#ifdef USING_SGTTY
  ioctl(fileno(f), TIOCSETP, &saved_tio);
#endif
  /* Restore the signal handler. */
  (void)signal(SIGINT, old_handler);
  /* Remove newline from the passphrase. */
  if (strchr(buf, '\n'))
    *strchr(buf, '\n') = 0;
  /* Allocate a copy of the passphrase. */
  //cp = strdup(buf);
  cp = (char *)xmalloc(sizeof(buf));
  cp = strcat(cp, buf);


  /* Clear the buffer so we don\'t leave copies of the passphrase laying
     around. */
  memset(buf, 0, sizeof(buf));
  /* Close the file. */
  if (f != stdin)
    fclose(f);
  return cp;
    
}

#else // USETERM


#ifdef HAVE_GETPASS

/*
 * the simple way
 */
char *read_passphrase(const char *prompt, char *cp) {
  cp = getpass(prompt);
  return cp;
}

#else // HAVE_GETPASS

char *read_passphrase(const char *prompt, char *cp) {
  fprintf(stderr, "%s", prompt);
  fflush(stderr);
  cp = (char *)xmalloc(1024);
  scanf("%s", cp);
  return cp;
}

#endif // HAVE_GETPASS


#endif // USETERM
